// SADemoTest.cpp
//
// Kenn Scribner, EnduraSoft
//
// Console test driver for the safearray test/demo object
//
#include <iostream>
#include <atlbase.h>
#include "..\SADemo\CComVector.h"
#include "..\SADemo\SADemo.h"

int main(int argc, char* argv[] )
{
	// Start COM
	CoInitialize(NULL);

	HRESULT hr = S_OK;
	try {
		// Create the object
		CComPtr<IArrayExample> spSAObject;
		hr = spSAObject.CoCreateInstance(__uuidof(ArrayExample));
		if ( FAILED(hr) ) throw hr;

		// Object created, print banner
		std::cout << "Safearray test object created." << std::endl;

		// Create storage for SAFEARRAY VARIANT
		CComVariant vt;
		hr = spSAObject->getListFromEnterprise(CComBSTR("nada"),&vt);
		if ( FAILED(hr) ) throw hr;

		// List retrieved, print banner
		std::cout << "Safearray list retrieved." << std::endl;

		// Assert the variant type
		ATLASSERT(vt.vt == (VT_SAFEARRAY));
		if ( vt.vt == (VT_SAFEARRAY) ) {
			// Extract data
		    CComVectorData<BSTR> rg(vt.parray);
		    if( !rg ) throw E_UNEXPECTED;

			USES_CONVERSION;
			for( int i = 0; i < rg.Length(); i++ ) {
				// Print the element
				CComBSTR bstrData;
				bstrData.Attach(rg[i]); // will gc for us...
				std::cout << "  Array element " << i << ": '" << W2A(bstrData) << "'" << std::endl;
			} // for
		} // if
		else {
			// Nope...
			std::cout << "Retrieved data was not a safearray!" << std::endl;
		} // else
	} // try
	catch(...) {
		std::cout << "Error in SADemoTest" << std::endl;
	} // catch

	// Stop COM
	CoUninitialize();

	return hr;
}